@@ -2,22 +2,29 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import importlib
-from pyhocon import ConfigFactory
-from databuilder.models.dashboard.dashboard_metadata import DashboardMetadata
+from typing import (
+    Any, Dict, Iterator, Optional,
+)
+
+from pyhocon import ConfigFactory, ConfigTree
+
+from databuilder.extractor.base_extractor import Extractor
+from databuilder.extractor.dashboard.redash.redash_dashboard_utils import (
+    RedashPaginatedRestApiQuery, generate_dashboard_description, get_auth_headers, get_text_widgets,
+    get_visualization_widgets, sort_widgets,
+)
+from databuilder.extractor.restapi.rest_api_extractor import REST_API_QUERY, RestAPIExtractor
+from databuilder.models.dashboard.dashboard_chart import DashboardChart
 from databuilder.models.dashboard.dashboard_last_modified import DashboardLastModifiedTimestamp
+from databuilder.models.dashboard.dashboard_metadata import DashboardMetadata
 from databuilder.models.dashboard.dashboard_owner import DashboardOwner
 from databuilder.models.dashboard.dashboard_query import DashboardQuery
 from databuilder.models.dashboard.dashboard_table import DashboardTable
 from databuilder.models.table_metadata import TableMetadata
-from databuilder.extractor.base_extractor import Extractor
-from databuilder.rest_api.rest_api_query import RestApiQuery
 from databuilder.rest_api.base_rest_api_query import EmptyRestApiQuerySeed
-from databuilder.extractor.restapi.rest_api_extractor import RestAPIExtractor, REST_API_QUERY
-from databuilder.extractor.dashboard.redash.redash_dashboard_utils import \
-    get_auth_headers, get_text_widgets, get_visualization_widgets, sort_widgets, \
-    generate_dashboard_description, RedashPaginatedRestApiQuery
+from databuilder.rest_api.rest_api_query import RestApiQuery
 from databuilder.transformer.base_transformer import ChainedTransformer
-from databuilder.transformer.timestamp_string_to_epoch import TimestampStringToEpoch, FIELD_NAME as TS_FIELD_NAME
+from databuilder.transformer.timestamp_string_to_epoch import FIELD_NAME as TS_FIELD_NAME, TimestampStringToEpoch
 
 
 class TableRelationData:
@@ -26,15 +33,15 @@ class TableRelationData:
     It is used as the type returned by the (optional) table parser.
     """
 
-    def __init__(self, database, cluster, schema, name):
-        # type: (str, str, str, str) -> None
-
+    def __init__(self,
+                 database: str,
+                 cluster: str,
+                 schema: str,
+                 name: str) -> None:
         self._data = {'db': database, 'cluster': cluster, 'schema': schema, 'tbl': name}
 
     @property
-    def key(self):
-        # type: () -> str
-
+    def key(self) -> str:
         return TableMetadata.TABLE_KEY_FORMAT.format(**self._data)
 
 
@@ -68,8 +75,7 @@ class RedashDashboardExtractor(Extractor):
     DASHBOARD_GROUP_ID = 'redash'
     DASHBOARD_GROUP_NAME = 'Redash'
 
-    def init(self, conf):
-        # type: (ConfigTree) -> None
+    def init(self, conf: ConfigTree) -> None:
 
         # required configuration
         self._redash_base_url = conf.get_string(RedashDashboardExtractor.REDASH_BASE_URL_KEY)
@@ -89,15 +95,12 @@ class RedashDashboardExtractor(Extractor):
 
         self._extractor = self._build_extractor()
         self._transformer = self._build_transformer()
-        self._extract_iter = None
-
-    def _is_published_dashboard(self, record):
-        # type: Dict[str, Any] -> bool
+        self._extract_iter: Optional[Iterator[Any]] = None
 
+    def _is_published_dashboard(self, record: Dict[str, Any]) -> bool:
         return not (record['is_archived'] or record['is_draft'])
 
-    def _get_extract_iter(self):
-        # type: () -> Iterator[Any]
+    def _get_extract_iter(self) -> Iterator[Any]:
 
         while True:
             record = self._extractor.extract()
@@ -112,8 +115,8 @@ class RedashDashboardExtractor(Extractor):
             identity_data = {
                 'cluster': self._cluster,
                 'product': RedashDashboardExtractor.PRODUCT,
-                'dashboard_group_id': RedashDashboardExtractor.DASHBOARD_GROUP_ID,
-                'dashboard_id': record['dashboard_id']
+                'dashboard_group_id': str(RedashDashboardExtractor.DASHBOARD_GROUP_ID),
+                'dashboard_id': str(record['dashboard_id'])
             }
 
             dash_data = {
@@ -124,8 +127,7 @@ class RedashDashboardExtractor(Extractor):
                 'dashboard_name':
                     record['dashboard_name'],
                 'dashboard_url':
-                    '{redash}/dashboard/{slug}'
-                    .format(redash=self._redash_base_url, slug=record['slug']),
+                    f'{self._redash_base_url}/dashboards/{record["dashboard_id"]}',
                 'created_timestamp':
                     record['created_timestamp']
             }
@@ -154,7 +156,7 @@ class RedashDashboardExtractor(Extractor):
 
             for viz in viz_widgets:
                 query_data = {
-                    'query_id': viz.query_id,
+                    'query_id': str(viz.query_id),
                     'query_name': viz.query_name,
                     'url': self._redash_base_url + viz.query_relative_url,
                     'query_text': viz.raw_query
@@ -163,6 +165,15 @@ class RedashDashboardExtractor(Extractor):
                 query_data.update(identity_data)
                 yield DashboardQuery(**query_data)
 
+                chart_data = {
+                    'query_id': str(viz.query_id),
+                    'chart_id': str(viz.visualization_id),
+                    'chart_name': viz.visualization_name,
+                    'chart_type': viz.visualization_type,
+                }
+                chart_data.update(identity_data)
+                yield DashboardChart(**chart_data)
+
                 # if a table parser is provided, retrieve tables from this viz
                 if self._parse_tables:
                     for tbl in self._parse_tables(viz):
@@ -171,8 +182,7 @@ class RedashDashboardExtractor(Extractor):
             if len(table_keys) > 0:
                 yield DashboardTable(table_ids=list(table_keys), **identity_data)
 
-    def extract(self):
-        # type: () -> Any
+    def extract(self) -> Any:
 
         if not self._extract_iter:
             self._extract_iter = self._get_extract_iter()
@@ -181,12 +191,11 @@ class RedashDashboardExtractor(Extractor):
         except StopIteration:
             return None
 
-    def _build_restapi_query(self):
-        # type: () -> RestApiQuery
+    def _build_restapi_query(self) -> RestApiQuery:
 
         dashes_query = RedashPaginatedRestApiQuery(
             query_to_join=EmptyRestApiQuerySeed(),
-            url='{redash_api}/dashboards'.format(redash_api=self._api_base_url),
+            url=f'{self._api_base_url}/dashboards',
             params=self._get_default_api_query_params(),
             json_path='results[*].[id,name,slug,created_at,updated_at,is_archived,is_draft,user]',
             field_names=[
@@ -198,20 +207,18 @@ class RedashDashboardExtractor(Extractor):
 
         return RestApiQuery(
             query_to_join=dashes_query,
-            url='{redash_api}/dashboards/{{slug}}'.format(redash_api=self._api_base_url),
+            url=f'{self._api_base_url}/dashboards/{{dashboard_id}}',
             params=self._get_default_api_query_params(),
             json_path='widgets',
             field_names=['widgets'],
             skip_no_result=True
         )
 
-    def _get_default_api_query_params(self):
-        # type: () -> Dict[str, Any]
+    def _get_default_api_query_params(self) -> Dict[str, Any]:
 
         return {'headers': get_auth_headers(self._api_key)}
 
-    def _build_extractor(self):
-        # type: () -> RestAPIExtractor
+    def _build_extractor(self) -> RestAPIExtractor:
 
         extractor = RestAPIExtractor()
         rest_api_extractor_conf = ConfigFactory.from_dict({
@@ -220,8 +227,7 @@ class RedashDashboardExtractor(Extractor):
         extractor.init(rest_api_extractor_conf)
         return extractor
 
-    def _build_transformer(self):
-        # type: () -> ChainedTransformer
+    def _build_transformer(self) -> ChainedTransformer:
 
         transformers = []
 
@@ -240,7 +246,6 @@ class RedashDashboardExtractor(Extractor):
 
         return ChainedTransformer(transformers=transformers)
 
-    def get_scope(self):
-        # type: () -> str
+    def get_scope(self) -> str:
 
         return 'extractor.redash_dashboard'
