@@ -1,12 +1,12 @@
 # Copyright Contributors to the Amundsen project.
 # SPDX-License-Identifier: Apache-2.0
 
-from typing import Any, Dict, List, Optional
 import re
+from typing import List, Optional
 
-from databuilder.models.neo4j_csv_serde import Neo4jCsvSerializable, NODE_KEY, \
-    NODE_LABEL, RELATION_START_KEY, RELATION_START_LABEL, RELATION_END_KEY, \
-    RELATION_END_LABEL, RELATION_TYPE, RELATION_REVERSE_TYPE
+from databuilder.models.graph_node import GraphNode
+from databuilder.models.graph_relationship import GraphRelationship
+from databuilder.models.graph_serializable import GraphSerializable
 
 
 class Badge:
@@ -15,11 +15,16 @@ class Badge:
         self.category = category
 
     def __repr__(self) -> str:
-        return 'Badge({!r}, {!r})'.format(self.name,
-                                          self.category)
+        return f'Badge({self.name!r}, {self.category!r})'
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, Badge):
+            return NotImplemented
+        return self.name == other.name and \
+            self.category == other.category
 
 
-class BadgeMetadata(Neo4jCsvSerializable):
+class BadgeMetadata(GraphSerializable):
     """
     Badge model.
     """
@@ -32,19 +37,12 @@ class BadgeMetadata(Neo4jCsvSerializable):
     INVERSE_BADGE_RELATION_TYPE = 'BADGE_FOR'
 
     def __init__(self,
-                 db_name: str,
-                 schema: str,
                  start_label: str,  # Table, Dashboard, Column
                  start_key: str,
                  badges: List[Badge],
-                 cluster: str = 'gold',  # is this what we want as default for badges..?
                  ):
         self.badges = badges
 
-        self.db = db_name.lower()
-        self.schema = schema.lower()
-        self.cluster = cluster.lower()
-
         table_key_pattern = re.compile('[a-z]+://[a-zA-Z0-9_.-]+.[a-zA-Z0-9_.-]+/[a-zA-Z0-9_.-]+')
         dashboard_key_pattern = re.compile('[a-z]+_dashboard://[a-zA-Z0-9_.-]+.[a-zA-Z0-9_.-]+/[a-zA-Z0-9_.-]+')
         column_key_pattern = re.compile('[a-z]+://[a-zA-Z0-9_.-]+.[a-zA-Z0-9_.-]+/[a-zA-Z0-9_.-]+/[a-zA-Z0-9_.-]+')
@@ -65,14 +63,17 @@ class BadgeMetadata(Neo4jCsvSerializable):
         self._node_iter = iter(self.create_nodes())
         self._relation_iter = iter(self.create_relation())
 
-    def create_next_node(self) -> Optional[Dict[str, Any]]:
+    def __repr__(self) -> str:
+        return f'BadgeMetadata({self.start_label!r}, {self.start_key!r})'
+
+    def create_next_node(self) -> Optional[GraphNode]:
         # return the string representation of the data
         try:
             return next(self._node_iter)
         except StopIteration:
             return None
 
-    def create_next_relation(self) -> Optional[Dict[str, Any]]:
+    def create_next_relation(self) -> Optional[GraphRelationship]:
         try:
             return next(self._relation_iter)
         except StopIteration:
@@ -87,30 +88,35 @@ class BadgeMetadata(Neo4jCsvSerializable):
     def get_metadata_model_key(self) -> str:
         return self.start_key
 
-    def create_nodes(self) -> List[Dict[str, Any]]:
+    def create_nodes(self) -> List[GraphNode]:
         """
-        Create a list of Neo4j node records
+        Create a list of `GraphNode` records
         :return:
         """
         results = []
         for badge in self.badges:
             if badge:
-                results.append({
-                    NODE_KEY: self.get_badge_key(badge.name),
-                    NODE_LABEL: self.BADGE_NODE_LABEL,
-                    self.BADGE_CATEGORY: badge.category
-                })
+                node = GraphNode(
+                    key=self.get_badge_key(badge.name),
+                    label=self.BADGE_NODE_LABEL,
+                    attributes={
+                        self.BADGE_CATEGORY: badge.category
+                    }
+                )
+                results.append(node)
         return results
 
-    def create_relation(self) -> List[Dict[str, Any]]:
-        results = []
+    def create_relation(self) -> List[GraphRelationship]:
+        results: List[GraphRelationship] = []
         for badge in self.badges:
-            results.append({
-                RELATION_START_LABEL: self.start_label,
-                RELATION_END_LABEL: self.BADGE_NODE_LABEL,
-                RELATION_START_KEY: self.start_key,
-                RELATION_END_KEY: self.get_badge_key(badge.name),
-                RELATION_TYPE: self.BADGE_RELATION_TYPE,
-                RELATION_REVERSE_TYPE: self.INVERSE_BADGE_RELATION_TYPE,
-            })
+            relation = GraphRelationship(
+                start_label=self.start_label,
+                end_label=self.BADGE_NODE_LABEL,
+                start_key=self.start_key,
+                end_key=self.get_badge_key(badge.name),
+                type=self.BADGE_RELATION_TYPE,
+                reverse_type=self.INVERSE_BADGE_RELATION_TYPE,
+                attributes={}
+            )
+            results.append(relation)
         return results
