@@ -2,23 +2,32 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import copy
-from collections import namedtuple
-
-from typing import Any, Dict, Iterable, Iterator, List, Optional, Set, Union
+from typing import (
+    Any, Dict, Iterable, Iterator, List, Optional, Set, Union,
+)
 
+from databuilder.models.badge import Badge, BadgeMetadata
 from databuilder.models.cluster import cluster_constants
-from databuilder.models.neo4j_csv_serde import (
-    Neo4jCsvSerializable, NODE_LABEL, NODE_KEY, RELATION_START_KEY, RELATION_END_KEY, RELATION_START_LABEL,
-    RELATION_END_LABEL, RELATION_TYPE, RELATION_REVERSE_TYPE)
-from databuilder.publisher.neo4j_csv_publisher import UNQUOTED_SUFFIX
+from databuilder.models.graph_node import GraphNode
+from databuilder.models.graph_relationship import GraphRelationship
+from databuilder.models.graph_serializable import GraphSerializable
 from databuilder.models.schema import schema_constant
-from databuilder.models.badge import BadgeMetadata, Badge
 
 DESCRIPTION_NODE_LABEL_VAL = 'Description'
 DESCRIPTION_NODE_LABEL = DESCRIPTION_NODE_LABEL_VAL
 
 
-class TagMetadata(Neo4jCsvSerializable):
+def _format_as_list(tags: Union[List, str, None]) -> List:
+    if tags is None:
+        tags = []
+    if isinstance(tags, str):
+        tags = list(filter(None, tags.split(',')))
+    if isinstance(tags, list):
+        tags = [tag.lower().strip() for tag in tags]
+    return tags
+
+
+class TagMetadata(GraphSerializable):
     TAG_NODE_LABEL = 'Tag'
     TAG_KEY_FORMAT = '{tag}'
     TAG_TYPE = 'tag_type'
@@ -34,7 +43,7 @@ class TagMetadata(Neo4jCsvSerializable):
         self._name = name
         self._tag_type = tag_type
         self._nodes = iter([self.create_tag_node(self._name, self._tag_type)])
-        self._relations: Iterator[Dict[str, Any]] = iter([])
+        self._relations: Iterator[GraphRelationship] = iter([])
 
     @staticmethod
     def get_tag_key(name: str) -> str:
@@ -43,21 +52,24 @@ class TagMetadata(Neo4jCsvSerializable):
         return TagMetadata.TAG_KEY_FORMAT.format(tag=name)
 
     @staticmethod
-    def create_tag_node(name: str,
-                        tag_type: str = DEFAULT_TYPE
-                        ) -> Dict[str, str]:
-        return {NODE_LABEL: TagMetadata.TAG_NODE_LABEL,
-                NODE_KEY: TagMetadata.get_tag_key(name),
-                TagMetadata.TAG_TYPE: tag_type}
+    def create_tag_node(name: str, tag_type: str = DEFAULT_TYPE) -> GraphNode:
+        node = GraphNode(
+            key=TagMetadata.get_tag_key(name),
+            label=TagMetadata.TAG_NODE_LABEL,
+            attributes={
+                TagMetadata.TAG_TYPE: tag_type
+            }
+        )
+        return node
 
-    def create_next_node(self) -> Optional[Dict[str, Any]]:
+    def create_next_node(self) -> Optional[GraphNode]:
         # return the string representation of the data
         try:
             return next(self._nodes)
         except StopIteration:
             return None
 
-    def create_next_relation(self) -> Optional[Dict[str, Any]]:
+    def create_next_relation(self) -> Optional[GraphRelationship]:
         # We don't emit any relations for Tag ingestion
         try:
             return next(self._relations)
@@ -115,31 +127,30 @@ class DescriptionMetadata:
             return "_" + self._source + "_description"
 
     def __repr__(self) -> str:
-        return 'DescriptionMetadata({!r}, {!r})'.format(self._source, self._text)
+        return f'DescriptionMetadata({self._source!r}, {self._text!r})'
 
-    def get_node_dict(self,
-                      node_key: str
-                      ) -> Dict[str, str]:
-        return {
-            NODE_LABEL: self._label,
-            NODE_KEY: node_key,
-            DescriptionMetadata.DESCRIPTION_SOURCE: self._source,
-            DescriptionMetadata.DESCRIPTION_TEXT: self._text or '',
-        }
+    def get_node(self, node_key: str) -> GraphNode:
+        node = GraphNode(
+            key=node_key,
+            label=self._label,
+            attributes={
+                DescriptionMetadata.DESCRIPTION_SOURCE: self._source,
+                DescriptionMetadata.DESCRIPTION_TEXT: self._text
+            }
+        )
+        return node
 
-    def get_relation(self,
-                     start_node: str,
-                     start_key: str,
-                     end_key: str
-                     ) -> Dict[str, str]:
-        return {
-            RELATION_START_LABEL: start_node,
-            RELATION_END_LABEL: self._label,
-            RELATION_START_KEY: start_key,
-            RELATION_END_KEY: end_key,
-            RELATION_TYPE: DescriptionMetadata.DESCRIPTION_RELATION_TYPE,
-            RELATION_REVERSE_TYPE: DescriptionMetadata.INVERSE_DESCRIPTION_RELATION_TYPE
-        }
+    def get_relation(self, start_node: str, start_key: Any, end_key: Any) -> GraphRelationship:
+        relationship = GraphRelationship(
+            start_label=start_node,
+            start_key=start_key,
+            end_label=self._label,
+            end_key=end_key,
+            type=DescriptionMetadata.DESCRIPTION_RELATION_TYPE,
+            reverse_type=DescriptionMetadata.INVERSE_DESCRIPTION_RELATION_TYPE,
+            attributes={}
+        )
+        return relationship
 
 
 class ColumnMetadata:
@@ -147,7 +158,7 @@ class ColumnMetadata:
     COLUMN_KEY_FORMAT = '{db}://{cluster}.{schema}/{tbl}/{col}'
     COLUMN_NAME = 'name'
     COLUMN_TYPE = 'type'
-    COLUMN_ORDER = 'sort_order{}'.format(UNQUOTED_SUFFIX)  # int value needs to be unquoted when publish to neo4j
+    COLUMN_ORDER = 'sort_order'
     COLUMN_DESCRIPTION = 'description'
     COLUMN_DESCRIPTION_FORMAT = '{db}://{cluster}.{schema}/{tbl}/{col}/{description_id}'
 
@@ -156,7 +167,7 @@ class ColumnMetadata:
                  description: Union[str, None],
                  col_type: str,
                  sort_order: int,
-                 badges: Union[List[str], None] = None
+                 badges: Union[List[str], None] = None,
                  ) -> None:
         """
         TODO: Add stats
@@ -164,31 +175,22 @@ class ColumnMetadata:
         :param description:
         :param col_type:
         :param sort_order:
+        :param badges: Optional. Column level badges
         """
         self.name = name
         self.description = DescriptionMetadata.create_description_metadata(source=None,
                                                                            text=description)
         self.type = col_type
         self.sort_order = sort_order
-        if badges:
-            self.badges = [Badge(badge, 'column') for badge in badges]
-        else:
-            self.badges = []
+        formatted_badges = _format_as_list(badges)
+        self.badges = [Badge(badge, 'column') for badge in formatted_badges]
 
     def __repr__(self) -> str:
-        return 'ColumnMetadata({!r}, {!r}, {!r}, {!r}, {!r})'.format(self.name,
-                                                                     self.description,
-                                                                     self.type,
-                                                                     self.sort_order,
-                                                                     self.badges)
+        return f'ColumnMetadata({self.name!r}, {self.description!r}, {self.type!r}, ' \
+               f'{self.sort_order!r}, {self.badges!r})'
 
 
-# Tuples for de-dupe purpose on Database, Cluster, Schema. See TableMetadata docstring for more information
-NodeTuple = namedtuple('KeyName', ['key', 'name', 'label'])
-RelTuple = namedtuple('RelKeys', ['start_label', 'end_label', 'start_key', 'end_key', 'type', 'reverse_type'])
-
-
-class TableMetadata(Neo4jCsvSerializable):
+class TableMetadata(GraphSerializable):
     """
     Table metadata that contains columns. It implements Neo4jCsvSerializable so that it can be serialized to produce
     Table, Column and relation of those along with relationship with table and schema. Additionally, it will create
@@ -202,7 +204,7 @@ class TableMetadata(Neo4jCsvSerializable):
     TABLE_NODE_LABEL = 'Table'
     TABLE_KEY_FORMAT = '{db}://{cluster}.{schema}/{tbl}'
     TABLE_NAME = 'name'
-    IS_VIEW = 'is_view{}'.format(UNQUOTED_SUFFIX)  # bool value needs to be unquoted when publish to neo4j
+    IS_VIEW = 'is_view'
 
     TABLE_DESCRIPTION_FORMAT = '{db}://{cluster}.{schema}/{tbl}/{description_id}'
 
@@ -228,8 +230,8 @@ class TableMetadata(Neo4jCsvSerializable):
     TAG_TABLE_RELATION_TYPE = 'TAG'
 
     # Only for deduping database, cluster, and schema (table and column will be always processed)
-    serialized_nodes: Set[Any] = set()
-    serialized_rels: Set[Any] = set()
+    serialized_nodes_keys: Set[Any] = set()
+    serialized_rels_keys: Set[Any] = set()
 
     def __init__(self,
                  database: str,
@@ -264,7 +266,7 @@ class TableMetadata(Neo4jCsvSerializable):
         self.is_view = is_view
         self.attrs: Optional[Dict[str, Any]] = None
 
-        self.tags = TableMetadata.format_tags(tags)
+        self.tags = _format_as_list(tags)
 
         if kwargs:
             self.attrs = copy.deepcopy(kwargs)
@@ -273,15 +275,8 @@ class TableMetadata(Neo4jCsvSerializable):
         self._relation_iterator = self._create_next_relation()
 
     def __repr__(self) -> str:
-        return 'TableMetadata({!r}, {!r}, {!r}, {!r} ' \
-               '{!r}, {!r}, {!r}, {!r})'.format(self.database,
-                                                self.cluster,
-                                                self.schema,
-                                                self.name,
-                                                self.description,
-                                                self.columns,
-                                                self.is_view,
-                                                self.tags)
+        return f'TableMetadata({self.database!r}, {self.cluster!r}, {self.schema!r}, {self.name!r} ' \
+               f'{self.description!r}, {self.columns!r}, {self.is_view!r}, {self.tags!r})'
 
     def _get_table_key(self) -> str:
         return TableMetadata.TABLE_KEY_FORMAT.format(db=self.database,
@@ -289,8 +284,7 @@ class TableMetadata(Neo4jCsvSerializable):
                                                      schema=self.schema,
                                                      tbl=self.name)
 
-    def _get_table_description_key(self,
-                                   description: DescriptionMetadata) -> str:
+    def _get_table_description_key(self, description: DescriptionMetadata) -> str:
         return TableMetadata.TABLE_DESCRIPTION_FORMAT.format(db=self.database,
                                                              cluster=self.cluster,
                                                              schema=self.schema,
@@ -329,102 +323,113 @@ class TableMetadata(Neo4jCsvSerializable):
 
     @staticmethod
     def format_tags(tags: Union[List, str, None]) -> List:
-        if tags is None:
-            tags = []
-        if isinstance(tags, str):
-            tags = list(filter(None, tags.split(',')))
-        if isinstance(tags, list):
-            tags = [tag.lower().strip() for tag in tags]
+        return _format_as_list(tags)
 
-        return tags
-
-    def create_next_node(self) -> Union[Dict[str, Any], None]:
+    def create_next_node(self) -> Union[GraphNode, None]:
         try:
             return next(self._node_iterator)
         except StopIteration:
             return None
 
-    def _create_next_node(self) -> Iterator[Any]:  # noqa: C901
-
-        table_node = {NODE_LABEL: TableMetadata.TABLE_NODE_LABEL,
-                      NODE_KEY: self._get_table_key(),
-                      TableMetadata.TABLE_NAME: self.name,
-                      TableMetadata.IS_VIEW: self.is_view}
-        if self.attrs:
-            for k, v in self.attrs.items():
-                if k not in table_node:
-                    table_node[k] = v
-        yield table_node
+    def _create_next_node(self) -> Iterator[GraphNode]:
+        yield self._create_table_node()
 
         if self.description:
             node_key = self._get_table_description_key(self.description)
-            yield self.description.get_node_dict(node_key)
+            yield self.description.get_node(node_key)
 
-        # Create the table tag node
+        # Create the table tag nodes
         if self.tags:
             for tag in self.tags:
                 yield TagMetadata.create_tag_node(tag)
 
         for col in self.columns:
-            yield {
-                NODE_LABEL: ColumnMetadata.COLUMN_NODE_LABEL,
-                NODE_KEY: self._get_col_key(col),
-                ColumnMetadata.COLUMN_NAME: col.name,
-                ColumnMetadata.COLUMN_TYPE: col.type,
-                ColumnMetadata.COLUMN_ORDER: col.sort_order}
+            column_node = GraphNode(
+                key=self._get_col_key(col),
+                label=ColumnMetadata.COLUMN_NODE_LABEL,
+                attributes={
+                    ColumnMetadata.COLUMN_NAME: col.name,
+                    ColumnMetadata.COLUMN_TYPE: col.type,
+                    ColumnMetadata.COLUMN_ORDER: col.sort_order
+                }
+            )
+            yield column_node
 
             if col.description:
                 node_key = self._get_col_description_key(col, col.description)
-                yield col.description.get_node_dict(node_key)
+                yield col.description.get_node(node_key)
 
             if col.badges:
-                badge_metadata = BadgeMetadata(db_name=self._get_database_key(),
-                                               schema=self._get_schema_key(),
-                                               start_label=ColumnMetadata.COLUMN_NODE_LABEL,
-                                               start_key=self._get_col_key(col),
-                                               badges=col.badges,
-                                               cluster=self._get_cluster_key())
-                badge_nodes = badge_metadata.create_nodes()
-                for node in badge_nodes:
+                col_badge_metadata = BadgeMetadata(
+                    start_label=ColumnMetadata.COLUMN_NODE_LABEL,
+                    start_key=self._get_col_key(col),
+                    badges=col.badges)
+                for node in col_badge_metadata.create_nodes():
                     yield node
 
         # Database, cluster, schema
-        others = [NodeTuple(key=self._get_database_key(),
-                            name=self.database,
-                            label=TableMetadata.DATABASE_NODE_LABEL),
-                  NodeTuple(key=self._get_cluster_key(),
-                            name=self.cluster,
-                            label=TableMetadata.CLUSTER_NODE_LABEL),
-                  NodeTuple(key=self._get_schema_key(),
-                            name=self.schema,
-                            label=TableMetadata.SCHEMA_NODE_LABEL)
-                  ]
+        others = [
+            GraphNode(
+                key=self._get_database_key(),
+                label=TableMetadata.DATABASE_NODE_LABEL,
+                attributes={
+                    'name': self.database
+                }
+            ),
+            GraphNode(
+                key=self._get_cluster_key(),
+                label=TableMetadata.CLUSTER_NODE_LABEL,
+                attributes={
+                    'name': self.cluster
+                }
+            ),
+            GraphNode(
+                key=self._get_schema_key(),
+                label=TableMetadata.SCHEMA_NODE_LABEL,
+                attributes={
+                    'name': self.schema
+                }
+            )
+        ]
 
         for node_tuple in others:
-            if node_tuple not in TableMetadata.serialized_nodes:
-                TableMetadata.serialized_nodes.add(node_tuple)
-                yield {
-                    NODE_LABEL: node_tuple.label,
-                    NODE_KEY: node_tuple.key,
-                    'name': node_tuple.name
-                }
+            if node_tuple.key not in TableMetadata.serialized_nodes_keys:
+                TableMetadata.serialized_nodes_keys.add(node_tuple.key)
+                yield node_tuple
 
-    def create_next_relation(self) -> Union[Dict[str, Any], None]:
+    def _create_table_node(self) -> GraphNode:
+        table_attributes = {
+            TableMetadata.TABLE_NAME: self.name,
+            TableMetadata.IS_VIEW: self.is_view
+        }
+        if self.attrs:
+            for k, v in self.attrs.items():
+                if k not in table_attributes:
+                    table_attributes[k] = v
+
+        return GraphNode(
+            key=self._get_table_key(),
+            label=TableMetadata.TABLE_NODE_LABEL,
+            attributes=table_attributes
+        )
+
+    def create_next_relation(self) -> Union[GraphRelationship, None]:
         try:
             return next(self._relation_iterator)
         except StopIteration:
             return None
 
-    def _create_next_relation(self) -> Iterator[Any]:
-
-        yield {
-            RELATION_START_LABEL: TableMetadata.SCHEMA_NODE_LABEL,
-            RELATION_END_LABEL: TableMetadata.TABLE_NODE_LABEL,
-            RELATION_START_KEY: self._get_schema_key(),
-            RELATION_END_KEY: self._get_table_key(),
-            RELATION_TYPE: TableMetadata.SCHEMA_TABLE_RELATION_TYPE,
-            RELATION_REVERSE_TYPE: TableMetadata.TABLE_SCHEMA_RELATION_TYPE
-        }
+    def _create_next_relation(self) -> Iterator[GraphRelationship]:
+        schema_table_relationship = GraphRelationship(
+            start_key=self._get_schema_key(),
+            start_label=TableMetadata.SCHEMA_NODE_LABEL,
+            end_key=self._get_table_key(),
+            end_label=TableMetadata.TABLE_NODE_LABEL,
+            type=TableMetadata.SCHEMA_TABLE_RELATION_TYPE,
+            reverse_type=TableMetadata.TABLE_SCHEMA_RELATION_TYPE,
+            attributes={}
+        )
+        yield schema_table_relationship
 
         if self.description:
             yield self.description.get_relation(TableMetadata.TABLE_NODE_LABEL,
@@ -433,63 +438,66 @@ class TableMetadata(Neo4jCsvSerializable):
 
         if self.tags:
             for tag in self.tags:
-                yield {
-                    RELATION_START_LABEL: TableMetadata.TABLE_NODE_LABEL,
-                    RELATION_END_LABEL: TagMetadata.TAG_NODE_LABEL,
-                    RELATION_START_KEY: self._get_table_key(),
-                    RELATION_END_KEY: TagMetadata.get_tag_key(tag),
-                    RELATION_TYPE: TableMetadata.TABLE_TAG_RELATION_TYPE,
-                    RELATION_REVERSE_TYPE: TableMetadata.TAG_TABLE_RELATION_TYPE,
-                }
+                tag_relationship = GraphRelationship(
+                    start_label=TableMetadata.TABLE_NODE_LABEL,
+                    start_key=self._get_table_key(),
+                    end_label=TagMetadata.TAG_NODE_LABEL,
+                    end_key=TagMetadata.get_tag_key(tag),
+                    type=TableMetadata.TABLE_TAG_RELATION_TYPE,
+                    reverse_type=TableMetadata.TAG_TABLE_RELATION_TYPE,
+                    attributes={}
+                )
+                yield tag_relationship
 
         for col in self.columns:
-            yield {
-                RELATION_START_LABEL: TableMetadata.TABLE_NODE_LABEL,
-                RELATION_END_LABEL: ColumnMetadata.COLUMN_NODE_LABEL,
-                RELATION_START_KEY: self._get_table_key(),
-                RELATION_END_KEY: self._get_col_key(col),
-                RELATION_TYPE: TableMetadata.TABLE_COL_RELATION_TYPE,
-                RELATION_REVERSE_TYPE: TableMetadata.COL_TABLE_RELATION_TYPE
-            }
+            column_relationship = GraphRelationship(
+                start_label=TableMetadata.TABLE_NODE_LABEL,
+                start_key=self._get_table_key(),
+                end_label=ColumnMetadata.COLUMN_NODE_LABEL,
+                end_key=self._get_col_key(col),
+                type=TableMetadata.TABLE_COL_RELATION_TYPE,
+                reverse_type=TableMetadata.COL_TABLE_RELATION_TYPE,
+                attributes={}
+            )
+            yield column_relationship
 
             if col.description:
-                yield col.description.get_relation(ColumnMetadata.COLUMN_NODE_LABEL,
-                                                   self._get_col_key(col),
-                                                   self._get_col_description_key(col, col.description))
+                yield col.description.get_relation(
+                    ColumnMetadata.COLUMN_NODE_LABEL,
+                    self._get_col_key(col),
+                    self._get_col_description_key(col, col.description)
+                )
+
             if col.badges:
-                badge_metadata = BadgeMetadata(db_name=self._get_database_key(),
-                                               schema=self._get_schema_key(),
-                                               start_label=ColumnMetadata.COLUMN_NODE_LABEL,
+                badge_metadata = BadgeMetadata(start_label=ColumnMetadata.COLUMN_NODE_LABEL,
                                                start_key=self._get_col_key(col),
-                                               badges=col.badges,
-                                               cluster=self._get_cluster_key())
+                                               badges=col.badges)
                 badge_relations = badge_metadata.create_relation()
                 for relation in badge_relations:
                     yield relation
 
         others = [
-            RelTuple(start_label=TableMetadata.DATABASE_NODE_LABEL,
-                     end_label=TableMetadata.CLUSTER_NODE_LABEL,
-                     start_key=self._get_database_key(),
-                     end_key=self._get_cluster_key(),
-                     type=TableMetadata.DATABASE_CLUSTER_RELATION_TYPE,
-                     reverse_type=TableMetadata.CLUSTER_DATABASE_RELATION_TYPE),
-            RelTuple(start_label=TableMetadata.CLUSTER_NODE_LABEL,
-                     end_label=TableMetadata.SCHEMA_NODE_LABEL,
-                     start_key=self._get_cluster_key(),
-                     end_key=self._get_schema_key(),
-                     type=TableMetadata.CLUSTER_SCHEMA_RELATION_TYPE,
-                     reverse_type=TableMetadata.SCHEMA_CLUSTER_RELATION_TYPE)
+            GraphRelationship(
+                start_label=TableMetadata.DATABASE_NODE_LABEL,
+                end_label=TableMetadata.CLUSTER_NODE_LABEL,
+                start_key=self._get_database_key(),
+                end_key=self._get_cluster_key(),
+                type=TableMetadata.DATABASE_CLUSTER_RELATION_TYPE,
+                reverse_type=TableMetadata.CLUSTER_DATABASE_RELATION_TYPE,
+                attributes={}
+            ),
+            GraphRelationship(
+                start_label=TableMetadata.CLUSTER_NODE_LABEL,
+                end_label=TableMetadata.SCHEMA_NODE_LABEL,
+                start_key=self._get_cluster_key(),
+                end_key=self._get_schema_key(),
+                type=TableMetadata.CLUSTER_SCHEMA_RELATION_TYPE,
+                reverse_type=TableMetadata.SCHEMA_CLUSTER_RELATION_TYPE,
+                attributes={}
+            )
         ]
 
         for rel_tuple in others:
-            if rel_tuple not in TableMetadata.serialized_rels:
-                TableMetadata.serialized_rels.add(rel_tuple)
-                yield {
-                    RELATION_START_LABEL: rel_tuple.start_label,
-                    RELATION_END_LABEL: rel_tuple.end_label,
-                    RELATION_START_KEY: rel_tuple.start_key,
-                    RELATION_END_KEY: rel_tuple.end_key,
-                    RELATION_TYPE: rel_tuple.type,
-                    RELATION_REVERSE_TYPE: rel_tuple.reverse_type
-                }
+            if (rel_tuple.start_key, rel_tuple.end_key, rel_tuple.type) not in TableMetadata.serialized_rels_keys:
+                TableMetadata.serialized_rels_keys.add((rel_tuple.start_key, rel_tuple.end_key, rel_tuple.type))
+                yield rel_tuple
